//+------------------------------------------------------------------+
//|                                                     MACD-RSI.mq5 |
//|                                                         VDV Soft |
//|                                                 vdv_2001@mail.ru |
//+------------------------------------------------------------------+
#property copyright "VDV Soft"
#property link      "vdv_2001@mail.ru"
#property version   "1.00"

#include <MovingAverages.mqh>

#property indicator_separate_window
#property indicator_buffers 8
#property indicator_plots   3
//--- plot buffer 1
#property indicator_label1  "MACD Histogram" //"extremely overshoot [-2/8]"
#property indicator_type1   DRAW_HISTOGRAM
#property indicator_color1  Silver
#property indicator_style1  STYLE_SOLID
#property indicator_width1  2
//--- plot buffer 2
#property indicator_label2  "MACD Signal" //"extremely overshoot [-2/8]"
#property indicator_type2   DRAW_LINE
#property indicator_color2  Red
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1
//--- plot buffer 3
#property indicator_label3  "RSI" //"extremely overshoot [-2/8]"
#property indicator_type3   DRAW_LINE
#property indicator_color3  DodgerBlue
#property indicator_style3  STYLE_SOLID
#property indicator_width3  1
//--- input parameters
input int InpFastEMA=12;
input int InpSlowEMA=26;
input int InpSignalSMA=9;
//---- input parameters
input int InpRSIPeriod=14;
input double InpCaliber=4.0; //need for RSI caliber
//--- indicator buffers
double   ExtMacdBuffer[];
double   ExtSignalBuffer[];
double   ExtRSIBuffer[];
double   ExtPosBuffer[];
double   ExtNegBuffer[];
double   ExtFastMA[];
double   ExtSlowMA[];
//--- MA handles
int   HandFastMa;
int   HandSlowMa;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,ExtMacdBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,ExtSignalBuffer,INDICATOR_DATA);
   SetIndexBuffer(2,ExtRSIBuffer,INDICATOR_DATA);
   SetIndexBuffer(4,ExtPosBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(5,ExtNegBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(6,ExtFastMA,INDICATOR_CALCULATIONS);
   SetIndexBuffer(7,ExtSlowMA,INDICATOR_CALCULATIONS);
//---
//--- get MA handles
   HandFastMa=iMA(NULL,0,InpFastEMA,0,MODE_EMA,PRICE_CLOSE);
   HandSlowMa=iMA(NULL,0,InpSlowEMA,0,MODE_EMA,PRICE_CLOSE);
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//--- check for data
   if(rates_total<InpSignalSMA+InpSlowEMA)
      return(0);
//--- not all data may be calculated
   int calculated=BarsCalculated(HandFastMa);
   if(calculated<rates_total)
     {
      Print("Not all data of FastMa is calculated (",calculated,"bars ). Error",GetLastError());
      return(0);
     }
   calculated=BarsCalculated(HandSlowMa);
   if(calculated<rates_total)
     {
      Print("Not all data of SlowMa is calculated (",calculated,"bars ). Error",GetLastError());
      return(0);
     }
//--- we can copy not all data
   int to_copy;
   if(prev_calculated>rates_total || prev_calculated<0)
      to_copy=rates_total;
   else
     {
      to_copy=rates_total-prev_calculated;
      if(prev_calculated>0) to_copy++;
     }
//--- get Fast EMA buffer
   if(CopyBuffer(HandFastMa,0,0,to_copy,ExtFastMA)<=0)
     {
      Print("Getting fast EMA is failed! Error",GetLastError());
      return(0);
     }
//--- get SlowSMA buffer
   if(CopyBuffer(HandSlowMa,0,0,to_copy,ExtSlowMA)<=0)
     {
      Print("Getting slow SMA is failed! Error",GetLastError());
      return(0);
     }
//---
   int limit;
   if(prev_calculated==0)
      limit=InpSignalSMA+InpSlowEMA;
   else limit=prev_calculated-(InpSignalSMA+InpSlowEMA);
//--- calculate MACD
   for(int i=limit;i<rates_total;i++)
     {
      ExtMacdBuffer[i]=ExtFastMA[i]-ExtSlowMA[i];

     }
   SimpleMAOnBuffer(rates_total,prev_calculated,0,InpSignalSMA,ExtMacdBuffer,ExtSignalBuffer);
//=============================================================== RSI =
   if(limit<=InpRSIPeriod) limit=InpRSIPeriod+1;
   double rel,negative,positive;
   while(limit<rates_total)
     {
      double sumn=0,sump=0;
      if(limit==InpRSIPeriod+1)
        {
         //---- initial accumulation
         int k=2;
         while(k<=limit)
           {
            rel=ExtMacdBuffer[k]-ExtMacdBuffer[k-1];
            if(rel>0) sump+=rel;
            else      sumn-=rel;
            k++;
           }
         positive=sump/InpRSIPeriod;
         negative=sumn/InpRSIPeriod;
        }
      else
        {
         //---- smoothed moving average
         rel=ExtMacdBuffer[limit]-ExtMacdBuffer[limit-1];
         if(rel>0)
            sump=rel;
         else
            sumn=-rel;
         positive=(ExtPosBuffer[limit-1]*(InpRSIPeriod-1)+sump)/InpRSIPeriod;
         negative=(ExtNegBuffer[limit-1]*(InpRSIPeriod-1)+sumn)/InpRSIPeriod;
        }
      ExtPosBuffer[limit]=positive;
      ExtNegBuffer[limit]=negative;
      if(negative==0)
         ExtRSIBuffer[limit]=0.0;
      else
         ExtRSIBuffer[limit]=InpCaliber*_Point*(50.0-100.0/(1+positive/negative));
      limit++;
     }

//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
